@@ -17,15 +17,15 @@
 # Author: benderc
 ###############################################################################
 
-ddepn <- function(dat, phiorig=NULL, phi=NULL, th=0.5, inference="netga", outfile=NULL,
+ddepn <- function(dat, phiorig=NULL, phi=NULL, th=0.8, inference="netga", outfile=NULL,
                   multicores=FALSE, maxiterations=1000, p=500, q=0.3, m=0.8, P=NULL,
 				  usebics=TRUE, cores=2, 
 				  priortype="laplaceinhib",
-				  lambda=NULL, B=NULL, samplelambda=NULL, #"fixed",
+				  lambda=NULL, B=NULL, samplelambda=NULL,
 				  hmmiterations=100, fanin=4,
 				  gam=NULL,it=NULL,K=NULL,quantL=.5,quantBIC=.5,
-				  debug=FALSE,burnin=1000, thin=FALSE, plotresults=TRUE,
-				  always_sample_sf=FALSE,scale_lik=FALSE) {
+				  debug=0,burnin=500, thin=FALSE, plotresults=TRUE,
+				  always_sample_sf=FALSE,scale_lik=FALSE,allow.stim.off=FALSE) {
 	# get the experiments, i.e. the stimuli/inhibitor combinations
 	# works if format of dat is like:
 	# colnames contain the experiments in form STIMULUS_time
@@ -45,42 +45,13 @@ ddepn <- function(dat, phiorig=NULL, phi=NULL, th=0.5, inference="netga", outfil
 			stop("ERROR: Please specify argument lambda either as numeric value, NULL or NA.")	
 		}
 	}
-	exps <- unique(tmp[1,])
-	#stims <- sapply(exps, function(x) strsplit(x,"&")[[1]])
-	## create a stimuli list
-	allstim <- as.character(unique(unlist(sapply(exps, function(x) strsplit(x,"&")[[1]]))))
-	stimuli <- list()
-	#allstim <- NULL
-	for(i in 1:length(exps)) {
-		expsi <- exps[i]
-		stims <- as.character(sapply(expsi, function(x) strsplit(x,"&")[[1]]))
-		stimsid <- NULL
-		for(j in 1:length(stims)) {
-			el <- stims[j]
-			x <- match(el,rownames(dat))	
-			# or define a number 
-			if(any(is.na(x)))
-				x <- match(el,allstim)
-			names(x) <- el
-			stimsid <- c(stimsid, x)		
-		}
-		names(stimsid) <- stims
-		stimuli[[i]] <- stimsid	
-	}
-	# add the stimuli as dummy data rows, if they are missing
-	stimm <- match(unique(names(unlist(stimuli))),rownames(dat))
-	if(any(is.na(stimm))) {
-		xx <- unlist(stimuli)
-		xxmat <- unique(cbind(xx,names(xx)))
-		toattach <- matrix(0.0,nrow=nrow(xxmat),ncol=ncol(dat),dimnames=list(xxmat[,2],colnames(dat)))
-		## remove stimulus that is already there, to get the order right
-		prune <- stimm[!is.na(stimm)]
-		if(length(prune)>0)
-			dat <- dat[-prune,]
-		dat <- rbind(toattach,dat)
-	}
-	n <- nrow(dat)
-	phinames <- rownames(dat)
+	tmp2 <- addstimuli(dat)
+	dat <- tmp2$dat
+	stimuli <- tmp2$stimuli
+	rm(tmp2)
+	N <- nrow(dat)
+	NC <- ncol(dat)
+	V <- rownames(dat)
 	
 	## check if seeds are provided properly
 	if(!is.null(phi)) {
@@ -89,8 +60,8 @@ ddepn <- function(dat, phiorig=NULL, phi=NULL, th=0.5, inference="netga", outfil
 			if(class(phi)=="list" && length(phi)!=p) {
 				stop(paste("Error: length of seed network list must be the same as p =",p))
 			} else if(class(phi)=="matrix") {
-				if(dim(phi)!=c(nrow(dat),nrow(dat))) {
-					stop(paste("Error: dimension of seed network must be",nrow(dat),"x",nrow(dat),"."))
+				if(dim(phi)!=c(N,N)) {
+					stop(paste("Error: dimension of seed network must be",N,"x",N,"."))
 				}
 				tmp <- vector("list",p)
 				for(i in 1:p)
@@ -98,7 +69,7 @@ ddepn <- function(dat, phiorig=NULL, phi=NULL, th=0.5, inference="netga", outfil
 				phi <- tmp
 				rm(tmp)
 			} else if(is.null(phi)) {
-				phi <- matrix(0, nrow=nrow(dat), ncol=ncol(dat), dimnames=list(rownames(dat),rownames(dat)))
+				phi <- matrix(0, nrow=N, ncol=NC, dimnames=list(rownames(dat),rownames(dat)))
 				tmp <- vector("list",p)
 				for(i in 1:p)
 					tmp[[i]] <- phi
@@ -107,17 +78,17 @@ ddepn <- function(dat, phiorig=NULL, phi=NULL, th=0.5, inference="netga", outfil
 			} else {
 				stop("Error: please provide either a list of or a single seed network in argument phi, or leave it as NULL.")
 			}
-			V <- rownames(dat)
-			tps <- unique(sapply(colnames(dat), function(x) strsplit(x,"_")[[1]][2]))
-			reps <- table(sub("_[0-9].*$","",colnames(dat))) / length(tps)
+			#V <- rownames(dat)
+			tps <- unique(sapply(cols, function(x) strsplit(x,"_")[[1]][2]))
+			reps <- table(sub("_[0-9].*$","",cols)) / length(tps)
 			X <- vector("list",p)
 			for(i in 1:p) {
 				X[[i]] <- phi[[i]]
 			}
 			if(multicores) {
-				P <- mclapply(X, getfirstphi, dat=dat,stimuli=stimuli,V=V,tps=tps,reps=reps,hmmiterations=hmmiterations,lambda=lambda,B=B,Z=Z,fanin=fanin,gam=gam,it=it,K=K,priortype=priortype,scale_lik=scale_lik, mc.preschedule=FALSE,mc.cores=cores)		
+				P <- mclapply(X, getfirstphi, dat=dat,stimuli=stimuli,V=V,tps=tps,reps=reps,hmmiterations=hmmiterations,lambda=lambda,B=B,Z=Z,fanin=fanin,gam=gam,it=it,K=K,priortype=priortype,scale_lik=scale_lik, allow.stim.off=allow.stim.off, mc.preschedule=FALSE,mc.cores=cores)		
 			} else {
-				P <- lapply(X, getfirstphi, dat=dat,stimuli=stimuli,V=V,tps=tps,reps=reps,hmmiterations=hmmiterations,lambda=lambda,B=B,Z=Z,fanin=fanin,gam=gam,it=it,K=K,priortype=priortype,scale_lik=scale_lik)
+				P <- lapply(X, getfirstphi, dat=dat,stimuli=stimuli,V=V,tps=tps,reps=reps,hmmiterations=hmmiterations,lambda=lambda,B=B,Z=Z,fanin=fanin,gam=gam,it=it,K=K,priortype=priortype,scale_lik=scale_lik, allow.stim.off=allow.stim.off)
 			}
 		} else if(inference=="mcmc") {
 			## if a list of networks is given, create a named list that is used for mcmc_ddepn
@@ -131,8 +102,8 @@ ddepn <- function(dat, phiorig=NULL, phi=NULL, th=0.5, inference="netga", outfil
 			} else if(class(phi)=="matrix") {
 				## if a matrix is given, create a list of cores copies
 				## for cores independent mcmc runs.
-				if(dim(phi)!=c(nrow(dat),nrow(dat))) {
-					stop(paste("Error: dimension of seed network must be",nrow(dat),"x",nrow(dat),"."))
+				if(dim(phi)!=c(N,N)) {
+					stop(paste("Error: dimension of seed network must be",N,"x",N,"."))
 				}
 				if(multicores==TRUE) {
 					## create list of start nets, copy the 
@@ -190,11 +161,13 @@ ddepn <- function(dat, phiorig=NULL, phi=NULL, th=0.5, inference="netga", outfil
 						p=p,q=q,m=m,multicores=multicores,usebics=usebics,
 						cores=cores,lambda=lambda,B=B,Z=Z,hmmiterations=hmmiterations,
 						scorefile=scorefile,fanin=fanin,
-						gam=gam,it=it,K=K,quantL=quantL,quantBIC=quantBIC,priortype=priortype, plotresults=plotresults,scale_lik=scale_lik))
+						gam=gam,it=it,K=K,quantL=quantL,quantBIC=quantBIC,priortype=priortype,
+						plotresults=plotresults,scale_lik=scale_lik, allow.stim.off=allow.stim.off,
+						debug=debug))
 		P <- retnetga$P
 		scorestats <- retnetga$scorestats
 		rm(retnetga)
-		phi.activation.count <- phi.inhibition.count <- weights.tc <- matrix(0,nrow=nrow(dat),ncol=nrow(dat),dimnames=list(rownames(dat),rownames(dat))) 
+		phi.activation.count <- phi.inhibition.count <- weights.tc <- matrix(0,nrow=N,ncol=N,dimnames=list(rownames(dat),rownames(dat))) 
 		# now compare all graphs to the original
         result <- NULL
 		resultep <- NULL
@@ -213,7 +186,12 @@ ddepn <- function(dat, phiorig=NULL, phi=NULL, th=0.5, inference="netga", outfil
 					phi.inhibition.count=phi.inhibition.count,
 					phi.orig=phiorig, phi=NULL, weights=NULL,
 					weights.tc=weights.tc, stats=result, conf.act=conf.act,conf.inh=conf.inh,
-					stimuli=stimuli) #, resultep=resultep)
+					stimuli=stimuli,
+					quantBIC=quantBIC,quantL=quantL,q=q,m=m,usebics=usebics)
+					#p=p,q=q,m=m,multicores=multicores,usebics=usebics,
+					#cores=cores,lambda=lambda,B=B,Z=Z,hmmiterations=hmmiterations,
+					#fanin=fanin, gam=gam,it=it,K=K,quantL=quantL,quantBIC=quantBIC,priortype=priortype,
+					#scale_lik=scale_lik, allow.stim.off=allow.stim.off,debug=debug)
         ret <- get.phi.final(ret,th)
 		if(plotresults)
         	plotrepresult(ret,outfile)
@@ -227,7 +205,7 @@ ddepn <- function(dat, phiorig=NULL, phi=NULL, th=0.5, inference="netga", outfil
 				## networks
 				if(is.null(P)) {
 					P <- list()
-					phistart <- matrix(0, nrow=n, ncol=n, dimnames=list(phinames,phinames))
+					phistart <- matrix(0, nrow=N, ncol=N, dimnames=list(V,V))
 					for(cr in 1:cores) {
 						P[[cr]] <- list(phi=phistart)
 					}
@@ -246,17 +224,15 @@ ddepn <- function(dat, phiorig=NULL, phi=NULL, th=0.5, inference="netga", outfil
 						usebics=usebics, cores=cores, lambda=lambda, B=B, Z=Z, samplelambda=samplelambda,
 						hmmiterations=hmmiterations,fanin=fanin, gam=gam, it=it, K=K, burnin=burnin,
 						priortype=priortype, plotresults=plotresults,always_sample_sf=always_sample_sf,scale_lik=scale_lik,
+						allow.stim.off=allow.stim.off,debug=debug,
 						mc.preschedule=FALSE,mc.cores=cores)
-				### experimental
-				if(debug)
-					browser()
 				#retlist <- get.phi.final.mcmc(retlist, maxiterations, prob=.3333, qu=.99999)
 				##### end experimental
 			} else {
 				#phistart <- matrix(sample(c(0,1,2),n*n,replace=T), nrow=n, ncol=n, dimnames=list(phinames,phinames))
 				if(is.null(phi)) {
 					## create empty start net, if phi==NULL
-					phistart <- matrix(0, nrow=n, ncol=n, dimnames=list(phinames,phinames))
+					phistart <- matrix(0, nrow=N, ncol=N, dimnames=list(V,V))
 				} else {
 					## otherwise, take the first net from phi
 					if(class(phi)=="list")
@@ -268,7 +244,8 @@ ddepn <- function(dat, phiorig=NULL, phi=NULL, th=0.5, inference="netga", outfil
 						th=th, multicores=multicores, outfile=outfile, maxiterations=maxiterations,
 						usebics=usebics, cores=cores, lambda=lambda, B=B, Z=Z, samplelambda=samplelambda,
 						hmmiterations=hmmiterations,fanin=fanin, gam=gam, it=it, K=K, burnin=burnin,
-						priortype=priortype, plotresults=plotresults,always_sample_sf=always_sample_sf,scale_lik=scale_lik)
+						priortype=priortype, plotresults=plotresults,always_sample_sf=always_sample_sf,scale_lik=scale_lik,
+						allow.stim.off=allow.stim.off,debug=debug)
 				## convert the return value to a list, to keep it in the same format
 				## as for the multicore==TRUE case
 				retlist <- list()
@@ -306,3 +283,151 @@ ddepn <- function(dat, phiorig=NULL, phi=NULL, th=0.5, inference="netga", outfil
 	ret
 }
 
+## uses a returned object from netga or inhibMCMC and resumes the sampling/optimisation
+resume_ddepn <- function(ret,maxiterations=10000,outfile=NULL,th=0.8,plotresults=TRUE,debug=0,cores=NULL) {
+	## close all x11 connections
+	graphics.off()
+	if(is.null(ret$samplings)) {
+		if(is.null(cores))
+			cores <- 1
+		mc <- cores>1
+		iter <- nrow(ret$scorestats)
+		maxiterations <- iter + maxiterations
+		## restore inference state
+		p <- length(ret$P)
+		q <- ret$q
+		m <- ret$m
+		quantBIC <- ret$quantBIC
+		quantL <- ret$quantL
+		usebics <- ret$usebics
+		pp <- ret$P[[1]]
+		lambda <- pp$lambda
+		B <- pp$B
+		Z <- pp$Z
+		hmmiterations <- pp$hmmiterations
+		fanin <- pp$fanin
+		gam <- pp$gam
+		it <- pp$it
+		K <- pp$K
+		priortype <- pp$priortype
+		scale_lik <- pp$scale_lik
+		allow.stim.off <- pp$allow.stim.off
+		N <- nrow(pp$phi)
+		dat <- ret$dat
+		stimuli <- ret$stimuli
+		phiorig <- ret$phi.orig
+		P <- ret$P
+		if(!is.null(outfile)) {
+			bname <- basename(outfile)
+			dname <- dirname(outfile)
+			scorefile <- paste(dname,paste("score",sub("\\.pdf","",bname),".pdf",sep=""),sep="/")
+		} else {
+			scorefile <- NULL
+		}
+		stime <- system.time(retnetga <- netga(dat,stimuli,P=P,maxiterations=maxiterations,
+						p=length(ret$P),q=q,m=m,multicores=mc,usebics=usebics,
+						cores=cores,lambda=lambda,B=B,Z=Z,hmmiterations=hmmiterations,
+						scorefile=scorefile,fanin=fanin,
+						gam=gam,it=it,K=K,quantL=quantL,quantBIC=quantBIC,priortype=priortype,
+						plotresults=plotresults,scale_lik=scale_lik, allow.stim.off=allow.stim.off,
+						debug=debug,retobj=ret))
+		P <- retnetga$P
+		scorestats <- retnetga$scorestats
+		rm(retnetga)
+		phi.activation.count <- phi.inhibition.count <- weights.tc <- matrix(0,nrow=N,ncol=N,dimnames=list(rownames(dat),rownames(dat))) 
+		# now compare all graphs to the original
+		result <- NULL
+		resultep <- NULL
+		for(i in 1:length(P)) {
+			if(!is.null(phiorig))
+				result <- rbind(result, cbind(compare.graphs.tc(phiorig, P[[i]]$phi,ignore.type=FALSE),t(as.matrix(stime))))
+			#resultep <- rbind(resultep, cbind(compare.graphs.ep(phiorig, P[[i]]$phi,tps=1:50,stimuli=stimuli), t(as.matrix(stime))))
+			phi.activation.count[which(P[[i]]$phi==1)] <- phi.activation.count[which(P[[i]]$phi==1)] + 1
+			phi.inhibition.count[which(P[[i]]$phi==2)] <- phi.inhibition.count[which(P[[i]]$phi==2)] + 1
+			weights.tc <- weights.tc + detailed.to.simple.regulations(P[[i]]$phi)
+		}
+		conf.act <- round(phi.activation.count/length(P),digits=3)
+		conf.inh <- round(phi.inhibition.count/length(P),digits=3)
+		weights.tc <- round(weights.tc/length(P),digits=3)
+		ret <- list(dat=dat, phi.activation.count=phi.activation.count,
+				phi.inhibition.count=phi.inhibition.count,
+				phi.orig=phiorig, phi=NULL, weights=NULL,
+				weights.tc=weights.tc, stats=result, conf.act=conf.act,conf.inh=conf.inh,
+				stimuli=stimuli,
+				quantBIC=quantBIC,quantL=quantL,q=q,m=m,usebics=usebics)
+		ret <- get.phi.final(ret,th)
+		if(plotresults)
+			plotrepresult(ret,outfile)
+		ret[["P"]] <- P
+		ret[["scorestats"]] <- scorestats
+	} else {
+		if(!is.null(cores)) {
+			print("NOTE: Argument cores is derived from the previous run. Omitting the given function argument cores.")
+			Sys.sleep(3)
+		}
+		## find out if multicores have to be used
+		cores <- ncol(ret$ltraces)
+		mc <- cores>1
+		## get the actual networks and the output file list
+		X <- list()
+		for(cr in 1:cores) {
+			if(!is.null(outfile))
+				filename <- paste(sub("\\.pdf","",outfile),"_",cr,".pdf",sep="")
+			else
+				filename <- NULL
+			retobj <- ret$samplings[[cr]]
+			X[[cr]] <- list(phi=retobj$phi, outfile=filename, retobj=retobj)
+		}
+		rs <- ret$samplings[[1]]
+		usebics <- rs$posterior>0
+		## phi is not used in runmcmc
+		## outfile is not used in runmcmc
+		## samplelambda in bestmodel! DONE
+		## it in bestmodel ist für scalefree prior, iter als neues element für die iteration! DONE
+		## always_sample_sf in bestmodel! DONE
+		if(mc) {
+			retlist <- mclapply(X, runmcmc, dat=rs$dat, phiorig=rs$phi.orig, phi=NULL, stimuli=rs$stimuli,
+						th=th, multicores=mc, outfile=NULL, maxiterations=maxiterations,
+						usebics=usebics, cores=cores, lambda=rs$lambda, B=rs$B, Z=rs$Z, samplelambda=rs$samplelambda,
+						hmmiterations=rs$hmmiterations,fanin=rs$fanin, gam=rs$gam, it=rs$it, K=rs$K, burnin=rs$burnin,
+						priortype=rs$priortype, plotresults=plotresults,always_sample_sf=rs$always_sample_sf,scale_lik=rs$scale_lik,
+						allow.stim.off=rs$allow.stim.off,debug=debug,
+						mc.preschedule=FALSE,mc.cores=cores)
+		} else {
+			ret <- runmcmc(X[[1]],rs$dat,rs$phi.orig,phi=NULL,rs$stimuli,th,mc,outfile=NULL,maxiterations,
+						usebics,cores,rs$lambda,rs$B,Z=NULL,samplelambda=rs$samplelambda,rs$hmmiterations,rs$fanin,rs$gam,rs$it,rs$K,burnin=rs$burnin,
+						rs$priortype,plotresults=plotresults,always_sample_sf=rs$always_sample_sf,rs$scale_lik,rs$allow.stim.off,debug=debug)
+			retlist <- list()
+			retlist[[1]] <- ret	
+		}
+		## get the likelihood traces
+		ltraces <- as.matrix(sapply(retlist,function(x) x$stats[,"MAP"]))
+		## thinning: make sure ltraces has less than 10000 rows
+		## TODO: change such that only 10000 rows are recorded (i.e. in mcmc_ddepn),
+		## otherwise it doesn't help to save memory for the trace storage
+		if(nrow(ltraces)>10000 && thin == TRUE){
+			ss <- seq(1,nrow(ltraces),by=nrow(ltraces)/10000)
+			ltraces <-  as.matrix(ltraces[ss,])
+		}
+		colors <- rainbow(ncol(ltraces))
+		ret <- list(samplings=retlist,ltraces=ltraces)
+		## plot results
+		if(!is.null(outfile)) {
+			pdf(outfile,onefile=TRUE)
+		}
+		if(plotresults) {
+			plot(as.numeric(rownames(ltraces)),ltraces[,1],type="l",xlab="iteration",ylab="Score",ylim=range(ltraces,na.rm=TRUE),col=colors[1],main="Score traces")
+			if(ncol(ltraces)>1)
+				sapply(2:ncol(ltraces), function(j,ltraces,colors) lines(as.numeric(rownames(ltraces)),ltraces[,j],col=colors[j]), ltraces=ltraces, colors=colors)
+			## get the final network from all cores inferences and plot
+			for(netnr in 1:length(retlist)) {
+				ret2 <- retlist[[netnr]]
+				plotdetailed(ret2$phi,stimuli=ret2$stimuli,weights=ret2$weights)
+			}
+		}
+		if(!is.null(outfile))
+			dev.off()
+	}
+	return(ret)
+}
+
