@@ -209,6 +209,7 @@ tp.median <- function(dat) {
 
 # transform multiple edge types to only one edge type
 detailed.to.simple.regulations <- function(phi) {
-	phi[phi==2] <- 1
-	phi
+	ifelse(phi==0,0,1)
+	#phi[phi==2] <- 1
+	#phi
 }
