@@ -9,12 +9,12 @@ simulatedata <- function(phi, mu.bg=0, sd.bg=0.1,
 		mu.signal.a=1, sd.signal.a=0.5,
 		mu.signal.i=-1, sd.signal.i=0.5,
 		stimulus=sample(nrow(phi),2),TT=10,R.t=4,R.b=3,
-		plot=FALSE,stimuli=NULL){
+		plot=FALSE,stimuli=NULL, allow.stim.off=TRUE){
 	print(paste("Stimuli: ",paste(stimulus,rownames(phi)[stimulus],sep="->"),sep=" "))
 	tps <- 1:TT
 	reps <- R.t*R.b
 	#gammaposs <- uniquegammaposs(propagate.effect.simple(phi,1:TT,stimulus,reps))
-	gammaposs <- propagate.effect.simple(phi,stimulus=stimulus,stimuli=stimuli)
+	gammaposs <- propagate.effect.simple(phi,stimulus=stimulus,stimuli=stimuli, allow.stim.off=allow.stim.off)
 	gammax <- gammaposs[,sort(sample(1:ncol(gammaposs), TT, replace=T))]
 	colnames(gammax) <- tps
 	gammax <- expand.gamma(gammax,reps)
